# ../scripts/custom/gg_join_message/gg_join_message.py

# ============================================================================
# >> IMPORTS
# ============================================================================
# Eventscripts Imports
#   ES
from es import isbot
from es import getplayername
from es import getUseridList
from es import ServerVar

# GunGame Imports
#   Addons
from gungame51.core.addons.shortcuts import AddonInfo
#   Messaging
from gungame51.core.messaging.shortcuts import msg
#   SQL
from gungame51.core.sql.shortcuts import get_rank
#   Players
from gungame51.core.players.shortcuts import Player

# ============================================================================
# >> ADDON REGISTRATION/INFORMATION
# ============================================================================
info = AddonInfo()
info.name = 'gg_join_message'
info.title = 'GG Join Message'
info.author = 'Satoon101'
info.version = '1.1'
info.translations = ['gg_join_message']

# =============================================================================
# >> PUBLIC VARIABLE
# =============================================================================
ServerVar(info.name + '_version', info.version,
        info.title + ' Version').set(info.version)
ServerVar(info.name + '_version').makepublic()


# ============================================================================
# >> CLASSES
# ============================================================================
class _JoinMessages(set):
    '''Class used to store players that have joined'''

    def add(self, userid):
        '''Adds a userid to the set'''

        # Typecast the userid
        userid = int(userid)

        # Is the player's userid already in the set?
        if userid in self:

            # If so, do not add them to the set
            return

        # Is the player a Bot?
        if isbot(userid):

            # If so, do not add them to the set
            return

        # Get the player instance
        ggPlayer = Player(userid)

        # Get the player's wins
        wins = ggPlayer.wins

        # Get the player's ranking
        rank = get_rank(ggPlayer.steamid)

        # Is player ranked?
        if rank == -1:

            # Use PlayerJoin_noRank message
            message = 'PlayerJoin_noRank'

        # Player is ranked
        else:

            # Use "PlayerJoin" message
            message = 'PlayerJoin'

        # Send message to all players
        msg('#human', message,
            {'name': getplayername(userid),
            'wins': wins, 'rank': rank}, True)

        # Add the userid to the set
        super(_JoinMessages, self).add(userid)

# Get the _JoinMessages instance
already_sent = _JoinMessages()


# ============================================================================
# >> LOAD & UNLOAD
# ============================================================================
def load():
    '''Called when the script is loaded'''

    # Store all userids in already_sent, so as to not send all on the next map
    for userid in getUseridList():

        # Add the userid
        already_sent.add(userid)


def unload():
    '''Called when the script is unloaded'''

    # Clear the set
    already_sent.clear()


# ============================================================================
# >> GAME EVENTS
# ============================================================================
def player_activate(event_var):
    '''Called when a player is activated on the server'''

    # Add userid to already_sent
    already_sent.add(event_var['userid'])


def player_disconnect(event_var):
    '''Called when a player disconnects from the server'''

    # Remove player from already_sent
    already_sent.discard(int(event_var['userid']))
